@@ -380,7 +380,17 @@ class PythonSphinxMapper(SphinxMapperBase):
             )
             obj.url_root = self.url_root
 
-            lines = sphinx.util.docstrings.prepare_docstring(obj.docstring)
+            for child_data in data.get("children", []):
+                for child_obj in self.create_class(
+                    child_data, options=options, **kwargs
+                ):
+                    obj.children.append(child_obj)
+
+            # Some objects require children to establish their docstring
+            # or type annotations (eg classes with inheritance),
+            # so do this after all children have been created.
+            lines = obj.docstring.splitlines()
+            lines.append("")  # Add back the trailing newline that .splitlines removes
             if lines and "autodoc-process-docstring" in self.app.events.events:
                 self.app.emit(
                     "autodoc-process-docstring", cls.type, obj.name, None, None, lines
@@ -388,12 +398,6 @@ class PythonSphinxMapper(SphinxMapperBase):
             obj.docstring = "\n".join(lines)
             self._record_typehints(obj)
 
-            for child_data in data.get("children", []):
-                for child_obj in self.create_class(
-                    child_data, options=options, **kwargs
-                ):
-                    obj.children.append(child_obj)
-
             # Parser gives children in source order already
             if self.app.config.autoapi_member_order == "alphabetical":
                 obj.children.sort(key=operator.attrgetter("name"))
@@ -403,16 +407,28 @@ class PythonSphinxMapper(SphinxMapperBase):
             yield obj
 
     def _record_typehints(self, obj):
-        if isinstance(
-            obj, (PythonClass, PythonFunction, PythonMethod)
-        ) and not obj.obj.get("overloads"):
+        if (
+            isinstance(obj, (PythonClass, PythonFunction, PythonMethod))
+            and not obj.overloads
+        ):
             obj_annotations = {}
-            for _, name, annotation, _ in obj.obj["args"]:
+
+            include_return_annotation = True
+            obj_data = obj.obj
+            if isinstance(obj, PythonClass):
+                constructor = obj.constructor
+                if constructor:
+                    include_return_annotation = False
+                    obj_data = constructor.obj
+                else:
+                    return
+
+            for _, name, annotation, _ in obj_data["args"]:
                 if name and annotation:
                     obj_annotations[name] = annotation
 
-            return_annotation = obj.obj.get("return_annotation")
-            if return_annotation:
+            return_annotation = obj_data["return_annotation"]
+            if include_return_annotation and return_annotation:
                 obj_annotations["return"] = return_annotation
 
             self.app.env.autoapi_annotations[obj.id] = obj_annotations
